#include <stdio.h>

#include "linked-list.h"


// adds an element before the head node
int add_head(Node_t **head, int data)
{
	Node_t *temp = NULL;

	temp = (Node_t *) malloc(sizeof(Node_t));
	
	if (!temp) 
		return -1;

	temp->data = data;
	temp->next = *head;
	*head = temp;

	return 0;
}

// adds an element after the node
int add_after(Node_t **head, int data)
{
	if ((*head))
		return add_head( &(*head)->next, data);
	else
		return add_head(head, data);
}

// deletes an element before the head node
int del_head(Node_t **head)
{
	Node_t *temp = NULL;
	int data=0;

	if (*head)
	{
		temp = *head;
		*head = (*head)->next;

		data=temp->data;
		free(temp);

		return data;
	}
	else
		return -1;
}

// deletes an element after the node
int del_after(Node_t **head)
{
	if (*head && (*head)->next)
		return del_head( &(*head)->next );
	else
		return del_head(head);
}

// deletes all elements from this node on ..
int del_all(Node_t **head)
{
	while (*head)
		del_head(head);

	return 0;
}

// prints all the nodes (from this node on ..)
void print_node(Node_t *node)
{
	int flag_first = 1;

	while (node)
	{
		if (flag_first)
			flag_first = 0;
		else
			printf(" -> ");

		printf("%d", node->data);

		node=node->next;
	}

	printf("\n");
}

// goes X nodes forward (steps indicates the amount)
Node_t *next_node(Node_t *node, int steps)
{
	while (node && node->next && steps--)
		node=node->next;

	return node;
}

// counts the elements (from this node on ..)
int count_node(Node_t *node)
{
	int count = 0;

	while (node)
	{
		count++;
		node = node->next;
	}

	return count;
}

// Adding as mentioned in the code will not work.
// A function Node_t *Prev_node(Node_t *head, Node_t *temp) is needed, otherwise it will break the chain (see next diagram).
//
// Before executing function:
//
// A.data, A.next -> B.data, B.next -> C.data, C.next -> D.data, D.next -> E.data, E.next
// ^ head                              ^ temp
//
// After executing function:
//
// A.data, A.next -> B.data, B.next -> ????? D.data, D.next -> E.data, E.next
// ^ head                                    ^ temp
//
// If you want to delete an element you are pointing to in the middle of the Linked List,
// go one backwords and use del_after(&temp) to delete it. be aware, it is O(n) in runtime.